unit FPCFunctions;

interface

uses classes,math,types,sysutils;

  Function Explode(Source,sep:string):TstringList;
  Function Implode(Source:TstringList;sep:string):String;
  function Get_Board_Extents(lines:TstringList):TRect;
  Function StrSubt(Val1:string;val2:integer):string;
  Procedure MoveToOrigin(var List:TstringList);
  Procedure Rotate90(var List:TstringList);
  
implementation

Function Explode(Source,sep:string):TstringList;
var
  Sb:TstringList;
begin
  Sb := TstringList.Create;
  while pos(sep,source)<> 0 do
  begin
    sb.add(copy(source,1,pos(sep,source)-1));
    delete(source,1,pos(sep,source)-1);
    delete(source,1,length(sep));
  end;
  if length(source) > 0 then
    sb.add(source);
  result := sb;
end;

Function Implode(Source:TstringList;sep:string):String;
var
  sb:string;
  c:integer;
begin
  sb :='';
  for c:= 0 to source.Count -2 do
    sb := sb+ source[c]+sep;
  sb := sb+source[source.count-1];
  result := sb;
end;

function Get_Board_Extents(lines:TstringList):TRect;
var
  Line:TstringList;
  Sb:Trect;
  c:integer;
  x,y:int64;
begin
  sb.Left := high(sb.left);
  sb.Right := low(sb.right);
  sb.Top := low(sb.top);
  sb.Bottom := high(sb.bottom);
  c := 0;
  repeat
    inc(c);
  until pos('[board]',lines[c]) = 1;
  repeat
    if pos('  corner: ',lines[c])= 1 then
    begin
      Line := explode(lines[c],' ');
      x := strtoint(line[4]);
      y := strtoint(line[5]);
      sb.Top := max(sb.top,y);
      sb.Bottom := min(sb.Bottom,y);
      sb.Left := min(sb.Left,x);
      sb.Right := max(sb.Right,x);
      line.Free;
    end;
    inc(c);
  until (pos('[solder_mask_cutouts]',lines[c])=1);
  result := sb;
end;

Function StrSubt(Val1:string;val2:integer):string;
begin
  result := inttostr(strtoint(val1)-val2);
end;

Procedure MoveToOrigin(var List:TstringList);
var
  R:Trect;
  exploded:TstringList;
  c:integer;
begin
  R := get_Board_Extents(list);
  c:= 0;
  while list[c] <> '[board]' do
    inc(c);
  //handle board section
  while list[c] <> '[solder_mask_cutouts]' do
  begin
    if pos('  corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[5] := StrSubt(exploded[5],r.Bottom);
      exploded[4] := StrSubt(exploded[4],r.Left);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle solder_mask_cutouts section
  while list[c] <> '[parts]' do
  begin
    if pos('  corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[5] := StrSubt(exploded[5],r.Bottom);
      exploded[4] := StrSubt(exploded[4],r.left);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle parts section
  while list[c] <> '[nets]' do
  begin
    if pos('  pos: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[4] := StrSubt(exploded[4],r.Bottom);
      exploded[3] := StrSubt(exploded[3],r.left);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle nets section
  while list[c] <> '[texts]' do
  begin
    if pos('    vtx: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[7] := StrSubt(exploded[7],r.Bottom);
      exploded[6] := StrSubt(exploded[6],r.left);
      list[c] := implode(exploded,' ');
    end;
    if pos('    corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[7] := StrSubt(exploded[7],r.Bottom);
      exploded[6] := StrSubt(exploded[6],r.left);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle texts section
  while list[c] <> '[end]' do
  begin
    if pos('text: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      exploded[exploded.count-6] := StrSubt(exploded[exploded.count-6],r.Bottom);
      exploded[exploded.count-7] := StrSubt(exploded[exploded.count-7],r.left);;
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
end;


Procedure Rotate90(var List:TstringList);
var
  exploded:TstringList;
  temp:string;
  c:integer;
begin
  c:= 0;
  while list[c] <> '[board]' do
    inc(c);
  //handle board section
  while list[c] <> '[solder_mask_cutouts]' do
  begin
    if pos('  corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[5];
      exploded[5] := inttostr(Strtoint(exploded[4])*-1);
      exploded[4] := temp;
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle solder_mask_cutouts section
  while list[c] <> '[parts]' do
  begin
    if pos('  corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[5];
      exploded[5] := inttostr(Strtoint(exploded[4])*-1);
      exploded[4] := temp;
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle parts section
  while list[c] <> '[nets]' do
  begin
    if pos('  pos: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[4];
      exploded[4] := inttostr(Strtoint(exploded[3])*-1);
      exploded[3] := temp;
      exploded[6] := inttostr((strtoint(exploded[6])+90) mod 360);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle nets section
  while list[c] <> '[texts]' do
  begin
    if pos('    vtx: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[7];
      exploded[7] := inttostr(Strtoint(exploded[6])*-1);
      exploded[6] := temp;
      list[c] := implode(exploded,' ');
    end;
    if pos('    corner: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[7];
      exploded[7] := inttostr(Strtoint(exploded[6])*-1);
      exploded[6] := temp;
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
  //Handle texts section
  while list[c] <> '[end]' do
  begin
    if pos('text: ',list[c]) = 1 then
    begin
      exploded := explode(list[c],' ');
      temp := exploded[exploded.count-6];
      exploded[exploded.count-6] := inttostr(Strtoint(exploded[exploded.count-7])*-1);
      exploded[exploded.count-7] := temp;
      exploded[exploded.count-4] := inttostr((strtoint(exploded[exploded.count-4])+90) mod 360);
      list[c] := implode(exploded,' ');
    end;
    inc(c);
  end;
end;

end.
 